<!-- This file is generated by scripts/link-docs.mjs from docs/tinymist/frontend/neovim.typ. Do not edit manually. -->
# Tinymist Neovim Support for Typst

Run and configure tinymist in Neovim for Typst.

## Features

See [Tinymist Features](https://github.com/Myriad-Dreamin/tinymist#features) for a list of features.


## Finding Executable

To enable LSP, you must install `tinymist`. You can find `tinymist` by:

- Night versions available at [GitHub Actions](https://github.com/Myriad-Dreamin/tinymist/actions).

- Stable versions available at [GitHub Releases](https://github.com/Myriad-Dreamin/tinymist/releases). 

  If you are using the latest version of
  [typst-ts-mode](https://codeberg.org/meow_king/typst-ts-mode), then
  you can use command `typst-ts-lsp-download-binary` to download the latest
  stable binary of `tinymist` at `typst-ts-lsp-download-path`.

- Build from source by cargo.
  You can also compile and install **latest** `tinymist` by [Cargo](https://www.rust-lang.org/tools/install).

  ```bash
  cargo install --git https://github.com/Myriad-Dreamin/tinymist --locked tinymist
  ```


- (Recommended) Stable versions available via [mason.nvim](https://github.com/williamboman/mason.nvim).

  ```lua
  {
    "williamboman/mason.nvim",
    opts = {
      ensure_installed = {
        "tinymist",
      },
    },
  }
  ```

## Setup

This section shows you a minimal way to setup tinymist in [LazyVim](https://www.lazyvim.org/). We also have some tips for users of other distros.


Please see [Extra Settings](#extra-settings) for more configuration.

### Setup for [LazyVim](https://www.lazyvim.org/)

Copy or merge the two files to corresponding paths into `~/.config/nvim/`.

- [Autocmds file](https://github.com/Myriad-Dreamin/tinymist/tree/main/editors/neovim/config/autocmds.lua) will help associate the `.typ` file extension with the `typst` filetype.
- [Plugin file](https://github.com/Myriad-Dreamin/tinymist/tree/main/editors/neovim/plugins/tinymist.lua) will help start tinymist for buffers with the `typst` filetype.

Then, check and restart Neovim.

### Setup for [AstroVim](https://github.com/AstroNvim)

AstroNvim now uses tinymist by default. Please check the [setup script](https://github.com/AstroNvim/astrocommunity/tree/main/lua/astrocommunity/pack/typst).

### Setup for [coc.nvim](https://github.com/neoclide/coc.nvim)

You can edit the `coc-settings.json` by executing `:CocConfig`:

```json
{
  "languageserver": {
    "tinymist": {
      "command": "tinymist",
      "filetypes": ["typst"],
      "settings": { ... }
    }
  }
}
```

## Tips

### Working with Multiple-File Projects

The solution is a bit internal, which should get further improvement, but you can pin a main file by command.

```lua
-- pin the main file
vim.lsp.buf.execute_command({ command = 'tinymist.pinMain', arguments = { vim.api.nvim_buf_get_name(0) } })
-- unpin the main file
vim.lsp.buf.execute_command({ command = 'tinymist.pinMain', arguments = { nil } })
```

There is also a plan to support multiple-file projects with a workspace configuration, but I don’t know whether it is Neovim’s way, so it needs further discussion.

> [!NOTE]
> 
>   `tinymist.pinMain` is a stateful command, and tinymist doesn't remember it between sessions (closing and opening the editor).


## Troubleshooting

### tinymist does not start on creating/opening files

First, please check that tinymist starts when manually setting the filetype.

```
:set filetype=typst
```

If tinymist starts, that means you have not made correct association between the file extension and filetype. There should be some error messages related to this in your lspconfig.

Please associate the `.typ` file extension with the `typst` filetype to start tinymist on file create/open events.

```shell
autocmd BufNewFile,BufRead *.typ setfiletype typst
```

## Extra Settings

### Configuring Language Server

To configure the language server, you can edit the `opts.servers.tinymist.settings`. For example, if you want to export to PDF on typing and output files in `$root_dir/target` directory:

```lua
return {
  -- add tinymist to lspconfig
  {
    "neovim/nvim-lspconfig",
    opts = {
      servers = {
        tinymist = {
          settings = {
            exportPdf = "onType",
            outputPath = "$root/target/$dir/$name",
          }
        },
      },
    },
  },
}
```

See [Tinymist Server Configuration](https://github.com/Myriad-Dreamin/tinymist/tree/main/editors/neovim/Configuration.md) for references.

### Configuring Folding Range for Neovim Client

Enable LSP-based folding range with `kevinhwang91/nvim-ufo`:

```lua
return {
  { -- configure language servers
    "neovim/nvim-lspconfig",
    dependencies = "kevinhwang91/nvim-ufo", -- enable LSP-based folds
  },
}
```

You can copy or merge [lsp-folding.lua](https://github.com/Myriad-Dreamin/tinymist/tree/main/editors/neovim/plugins/lsp-folding.lua) to corresponding paths into `~/.config/nvim/` and restart Neovim.

## Contributing

You can submit issues or make PRs to [GitHub](https://github.com/Myriad-Dreamin/tinymist).
